#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Generate a skymap with equatorial grid"""

import numpy as np
from matplotlib import pyplot as plt
from matplotlib.collections import LineCollection
from skyfield.api import Star, load#, wgs84
from skyfield.data import hipparcos, stellarium
from skyfield.projections import build_stereographic_projection
from astroquery.simbad import Simbad
from astropy.coordinates import SkyCoord
import astropy.units as u
from astropy.wcs import WCS
import matplotlib.animation as animation
from matplotlib.animation import FuncAnimation, PillowWriter  # animation et export video
import locale # règles locales (noms des mois)

# =============================================================================
# --------------------paramètres à personnaliser :
# =============================================================================

plt.style.use("dark_background")
plt.rcParams['font.family'] = 'serif'
plt.rcParams['font.serif'] = ['Times New Roman']
locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')      # dates en français

pasJours = 5  # nombre de jours entre deux positions
couleur_mars = 'orange'
ts = load.timescale()
    # définition de la période de recherche des positions de Mars :
periode = ts.utc(2026, 8, range(1, 420, 3))
long_filante = 20  # nombre de dates de la ligne filante
FOV = 60.0  # étendue du champ de vision
MAG = 6.0  # magnitude maximale des étoiles
# =============================================================================
# Récupération des données astronomiques
# =============================================================================
# Query object from Simbad
OBJECT = "Regulus"  # objet au centre du graphique

TABLE = Simbad.query_object(OBJECT)
RA = TABLE['RA'][0]
DEC = TABLE['DEC'][0]
COORD = SkyCoord(f"{RA} {DEC}", unit=(u.hourangle, u.deg), frame='fk5')

t = ts.utc(2026, 12, 10, 20)
# An ephemeris from the JPL provides Sun and Earth positions.
eph = load('de421.bsp')  # (télé)chargement du fichier d'éphérides du JPL
earth = eph['earth']
mars = eph['mars']

# =============================================================================
# -------------- Coordonnées du fond étoilé et tracé --------------------
# =============================================================================

# Load constellation outlines from Stellarium
url = ('https://raw.githubusercontent.com/Stellarium/stellarium/master'
       '/skycultures/modern_st/constellationship.fab')

with load.open(url) as f:
    constellations = stellarium.parse_constellations(f)

edges = [edge for name, edges in constellations for edge in edges]
edges_star1 = [star1 for star1, star2 in edges]
edges_star2 = [star2 for star1, star2 in edges]

# The Hipparcos mission provides our star catalog.
with load.open(hipparcos.URL) as f:
    stars = hipparcos.load_dataframe(f)

# Center the chart on the specified object's position.
center = earth.at(t).observe(Star(ra_hours=COORD.ra.hour, dec_degrees=COORD.dec.degree))
projection = build_stereographic_projection(center)

# Compute the x and y coordinates that each star will have on the plot.
star_positions = earth.at(t).observe(Star.from_dataframe(stars))
stars['x'], stars['y'] = projection(star_positions)

# Create a True/False mask marking the stars bright enough to be included in our plot.
bright_stars = (stars.magnitude <= MAG)
magnitude = stars['magnitude'][bright_stars]
marker_size = (0.5 + MAG - magnitude) ** 2.0

# The constellation lines will each begin at the x,y of one star and end at the x,y of another.
xy1 = stars[['x', 'y']].loc[edges_star1].values
xy2 = stars[['x', 'y']].loc[edges_star2].values
lines_xy = np.rollaxis(np.array([xy1, xy2]), 1)

# Define the limit for the plotting area
angle = np.deg2rad(FOV / 2.0)
limit = np.tan(angle)  # Calculate limit based on the field of view


# =============================================================================
# ------------ Définition de la figure et du graphique ----------------------
# =============================================================================

# Build the figure with WCS axes
fig = plt.figure(figsize=[6, 6])
wcs = WCS(naxis=2)
wcs.wcs.crpix = [1, 1]
# wcs.wcs.cdelt = np.array([-FOV / 360, FOV / 360])
wcs.wcs.cdelt = np.array([-360 / np.pi, 360 / np.pi])
wcs.wcs.crval = [COORD.ra.deg, COORD.dec.deg]
wcs.wcs.ctype = ["RA---STG", "DEC--STG"]
ax = fig.add_subplot(111, projection=wcs)

# Draw the constellation lines
# ax.add_collection(LineCollection(lines_xy, colors='azure', linewidths=0.5,
#                                  linestyle='-'))

# Draw the stars
# ax.scatter(stars['x'][bright_stars], stars['y'][bright_stars],
#            s=marker_size, color='white', zorder=2)

angle = np.pi - FOV / 360.0 * np.pi
limit = np.sin(angle) / (1.0 - np.cos(angle))

# Add RA/Dec grid lines
ax.coords.grid(True, color='white', linestyle='dotted')

# Set the coordinate grid
ax.coords[0].set_axislabel('ascension droite (heures)')
ax.coords[1].set_axislabel('déclinaison (degrés)')
# ax.coords[0].set_major_formatter('hh:mm')
# ax.coords[1].set_major_formatter('dd:mm:ss')

# =============================================================================
#   --------------------mouvement apparent de Mars ----------------
# =============================================================================

deb = periode[0].utc_strftime('%d %b %Y')
fin = periode[-1].utc_strftime('%d %b %Y')

    # calcul des coordonnées de Mars durant la période :
ra, dec = projection((earth).at(periode).observe(mars))
x, y = ra, dec

    # définitions initiales de la ligne et du nuage de points :

line, = ax.plot([], [], c = couleur_mars, lw=0.8)
point, = ax.plot([], [], 'o', c = couleur_mars, mec='k', lw=0.2)
    # texte pour la date de chaque position :
texte = ax.text(0.02, 0.98,'', va='top', fontweight='bold', fontsize=12,
                transform=ax.transAxes, color = couleur_mars)

# =============================================================================
# ---------------------- Fonctions ----------------------------------
# =============================================================================
def fixe():
    """
    Fonction qui initialise la figure. Appelée 1 fois au début.
    """
    ax.set_xlim(-limit, limit)
    ax.set_ylim(-limit, limit)
    line.set_data([],[])
    texte.set_text('')

    # Draw the constellation lines
    ax.add_collection(LineCollection(lines_xy, colors='azure', linewidths=0.5,
                                     linestyle='-'))

    # Draw the stars
    ax.scatter(stars['x'][bright_stars], stars['y'][bright_stars],
               s=marker_size, color='white', zorder=2)

def fonction_animation(i):
    """
    Fonction qui met à jour les éléments variables.
    """
    ax.set_title(f'Rétrogradation de Mars \n du {deb} au {fin}', color='white', y=1.04)
    ax.set_aspect('equal')
    line.set_data(x[i-long_filante :i], y[i-long_filante :i])
    point.set_data([x[i],x[i]], [y[i], y[i]])
    texte.set_text(periode[i].utc_datetime().strftime('%d %b'))
    return line, point, texte

    # création de l'animation (pour la fenêtre externe de matplotlib):
ani = FuncAnimation(fig, fonction_animation, interval=50, blit=False,
                    init_func=fixe,
                    repeat=False,  # n'agit pas sur le fichier GIF généré
                    frames=len(periode)
                    )

    # sauvegarde en format GIF animé :
# ani.save("retrogradation_Mars.gif", dpi=300,
#             writer = PillowWriter(fps=4)
#           )

    # sauvegarde en format vidéo :
Writer = animation.writers['ffmpeg']
writer = Writer(fps=10, bitrate=-1)
ani.save("retrogradation_MarsFilante.mp4", dpi=300,
          writer = writer
          )

plt.show()
